@@ -7,6 +7,7 @@
 
 index((r_open), 0, r_session, 'R').
 index((r_open), 1, r_session, 'R').
+index((r_start), 0, r_session, 'R').
 index((r_close), 0, r_session, 'R').
 index((r_close), 1, r_session, 'R').
 index((r_in), 1, r_session, 'R').
@@ -41,6 +42,7 @@ index((r_bin), 1, r_session, 'R').
 index((r_bin_version), 1, r_session, 'R').
 index((r_bin_version), 2, r_session, 'R').
 index((r_verbosity), 1, r_session, 'R').
+index((<-), 2, r_session, 'R').
 index((foreach), 2, aggregate, aggregate).
 index((aggregate), 3, aggregate, aggregate).
 index((aggregate), 4, aggregate, aggregate).
@@ -50,6 +52,7 @@ index((free_variables), 4, aggregate, aggregate).
 index((am_match), 1, am_match, am_match).
 index((am_compile), 1, am_match, am_match).
 index((am_bagof), 4, am_match, am_match).
+index((ansi_format), 3, ansi_term, ansi_term).
 index((include), 3, apply, apply).
 index((exclude), 3, apply, apply).
 index((partition), 4, apply, apply).
@@ -118,6 +121,8 @@ index((setup_and_call_cleanup), 3, backward_compatibility, backcomp).
 index((setup_and_call_cleanup), 4, backward_compatibility, backcomp).
 index((merge), 3, backward_compatibility, backcomp).
 index((merge_set), 3, backward_compatibility, backcomp).
+index((base32), 2, base32, base32).
+index((base32), 3, base32, base32).
 index((base64), 2, base64, base64).
 index((base64), 3, base64, base64).
 index((please), 2, bim, bim).
@@ -267,6 +272,11 @@ index((fast_write_to_string), 3, fastrw, fastrw).
 index((can_open_file), 2, files, files).
 index((chdir), 1, files, files).
 index((set_time_file), 3, files_ex, filesex).
+index((link_file), 3, files_ex, filesex).
+index((relative_file_name), 3, files_ex, filesex).
+index((directory_file_path), 3, files_ex, filesex).
+index((copy_file), 2, files_ex, filesex).
+index((make_directory_path), 1, files_ex, filesex).
 index((reset_gensym), 0, gensym, gensym).
 index((reset_gensym), 1, gensym, gensym).
 index((gensym), 2, gensym, gensym).
@@ -281,6 +291,7 @@ index((list_to_heap), 2, heaps, heaps).
 index((merge_heaps), 3, heaps, heaps).
 index((min_of_heap), 3, heaps, heaps).
 index((min_of_heap), 5, heaps, heaps).
+index((singleton_heap), 3, heaps, heaps).
 index((help), 1, online_help, help).
 index((help), 0, online_help, help).
 index((apropos), 1, online_help, help).
@@ -288,6 +299,10 @@ index((predicate), 5, help_index, helpidx).
 index((section), 4, help_index, helpidx).
 index((function), 3, help_index, helpidx).
 index((load_hotfixes), 1, prolog_hotfix, hotfix).
+index((iso_639_2), 2, iso_639, iso_639).
+index((iso_639_3), 2, iso_639, iso_639).
+index((iso_639), 2, iso_639, iso_639).
+index((isub), 4, isub, isub).
 index((listing), 0, prolog_listing, listing).
 index((listing), 1, prolog_listing, listing).
 index((portray_clause), 1, prolog_listing, listing).
@@ -298,6 +313,8 @@ index((append), 3, lists, lists).
 index((prefix), 2, lists, lists).
 index((select), 3, lists, lists).
 index((selectchk), 3, lists, lists).
+index((select), 4, lists, lists).
+index((selectchk), 4, lists, lists).
 index((nextto), 3, lists, lists).
 index((delete), 3, lists, lists).
 index((nth0), 3, lists, lists).
@@ -428,6 +445,7 @@ index((process_kill), 2, process, process).
 index((clause_info), 4, prolog_clause, prolog_clause).
 index((predicate_name), 2, prolog_clause, prolog_clause).
 index((clause_name), 2, prolog_clause, prolog_clause).
+index((prolog_history), 1, prolog_history, prolog_history).
 index((prolog_server), 2, prolog_server, prolog_server).
 index((prolog_read_source_term), 4, prolog_source, prolog_source).
 index((prolog_open_source), 2, prolog_source, prolog_source).
@@ -435,6 +453,7 @@ index((prolog_close_source), 1, prolog_source, prolog_source).
 index((prolog_canonical_source), 2, prolog_source, prolog_source).
 index((file_name_on_path), 2, prolog_source, prolog_source).
 index((file_alias_path), 2, prolog_source, prolog_source).
+index((path_segments_atom), 2, prolog_source, prolog_source).
 index((get_prolog_backtrace), 2, prolog_stack, prolog_stack).
 index((get_prolog_backtrace), 3, prolog_stack, prolog_stack).
 index((print_prolog_backtrace), 2, prolog_stack, prolog_stack).
@@ -445,16 +464,17 @@ index((xref_defined), 3, prolog_xref, prolog_xref).
 index((xref_definition_line), 2, prolog_xref, prolog_xref).
 index((xref_exported), 2, prolog_xref, prolog_xref).
 index((xref_module), 2, prolog_xref, prolog_xref).
+index((xref_uses_file), 3, prolog_xref, prolog_xref).
 index((xref_op), 2, prolog_xref, prolog_xref).
 index((xref_clean), 1, prolog_xref, prolog_xref).
 index((xref_current_source), 1, prolog_xref, prolog_xref).
 index((xref_done), 2, prolog_xref, prolog_xref).
 index((xref_built_in), 1, prolog_xref, prolog_xref).
-index((xref_expand), 2, prolog_xref, prolog_xref).
 index((xref_source_file), 3, prolog_xref, prolog_xref).
 index((xref_source_file), 4, prolog_xref, prolog_xref).
 index((xref_public_list), 4, prolog_xref, prolog_xref).
-index((xref_public_list), 5, prolog_xref, prolog_xref).
+index((xref_public_list), 6, prolog_xref, prolog_xref).
+index((xref_public_list), 7, prolog_xref, prolog_xref).
 index((xref_meta), 2, prolog_xref, prolog_xref).
 index((xref_hook), 1, prolog_xref, prolog_xref).
 index((xref_used_class), 2, prolog_xref, prolog_xref).
@@ -464,6 +484,9 @@ index((protobuf_message), 3, protobufs, protobufs).
 index((phrase_from_file), 2, pure_input, pure_input).
 index((phrase_from_file), 3, pure_input, pure_input).
 index((stream_to_lazy_list), 2, pure_input, pure_input).
+index((pwp_files), 2, pwp, pwp).
+index((pwp_stream), 3, pwp, pwp).
+index((pwp_xml), 3, pwp, pwp).
 index((load_foreign_files), 0, qp_foreign, qpforeign).
 index((load_foreign_files), 2, qp_foreign, qpforeign).
 index((load_foreign_files), 3, qp_foreign, qpforeign).
@@ -493,7 +516,6 @@ index((sign), 2, quintus, quintus).
 index((sqrt), 2, quintus, quintus).
 index((genarg), 3, quintus, quintus).
 index((mode), 1, quintus, quintus).
-index((public), 1, quintus, quintus).
 index((no_style_check), 1, quintus, quintus).
 index((otherwise), 0, quintus, quintus).
 index((simple), 1, quintus, quintus).
@@ -517,6 +539,7 @@ index((randseq), 3, random, random).
 index((randset), 3, random, random).
 index((getrand), 1, random, random).
 index((setrand), 1, random, random).
+index((random_permutation), 2, random, random).
 index((rb_new), 1, rbtrees, rbtrees).
 index((rb_empty), 1, rbtrees, rbtrees).
 index((rb_lookup), 3, rbtrees, rbtrees).
@@ -548,6 +571,24 @@ index((ord_list_to_rbtree), 2, rbtrees, rbtrees).
 index((is_rbtree), 1, rbtrees, rbtrees).
 index((rb_size), 2, rbtrees, rbtrees).
 index((rb_in), 3, rbtrees, rbtrees).
+index((load_rdf), 2, rdf, rdf).
+index((load_rdf), 3, rdf, rdf).
+index((xml_to_rdf), 3, rdf, rdf).
+index((process_rdf), 3, rdf, rdf).
+index((load_rdf_ntriples), 2, rdf_ntriples, rdf_ntriples).
+index((rdf_ntriple_part), 4, rdf_ntriples, rdf_ntriples).
+index((xml_to_plrdf), 3, rdf_parser, rdf_parser).
+index((element_to_plrdf), 3, rdf_parser, rdf_parser).
+index((make_rdf_state), 3, rdf_parser, rdf_parser).
+index((rdf_modify_state), 3, rdf_parser, rdf_parser).
+index((rdf_name_space), 1, rdf_parser, rdf_parser).
+index((rdf_triples), 2, rdf_triple, rdf_triple).
+index((rdf_triples), 3, rdf_triple, rdf_triple).
+index((rdf_reset_ids), 0, rdf_triple, rdf_triple).
+index((rdf_start_file), 2, rdf_triple, rdf_triple).
+index((rdf_end_file), 1, rdf_triple, rdf_triple).
+index((anon_prefix), 1, rdf_triple, rdf_triple).
+index((rdf_write_xml), 2, rdf_write, rdf_write).
 index((readln), 1, readln, readln).
 index((readln), 2, readln, readln).
 index((readln), 5, readln, readln).
@@ -559,6 +600,9 @@ index((read_file_to_codes), 3, read_util, readutil).
 index((read_file_to_terms), 3, read_util, readutil).
 index((record), 1, record, record).
 index((current_record), 2, record, record).
+index((rewrite), 2, rewrite, rewrite).
+index((rew_term_expansion), 2, rewrite, rewrite).
+index((rew_goal_expansion), 2, rewrite, rewrite).
 index((rlimit), 3, clib_rlimit, rlimit).
 index((setting), 4, settings, settings).
 index((setting), 2, settings, settings).
@@ -573,6 +617,38 @@ index((current_setting), 1, settings, settings).
 index((setting_property), 2, settings, settings).
 index((list_settings), 0, settings, settings).
 index((convert_setting_text), 3, settings, settings).
+index((load_sgml_file), 2, sgml, sgml).
+index((load_xml_file), 2, sgml, sgml).
+index((load_html_file), 2, sgml, sgml).
+index((load_structure), 3, sgml, sgml).
+index((load_dtd), 2, sgml, sgml).
+index((load_dtd), 3, sgml, sgml).
+index((dtd), 2, sgml, sgml).
+index((dtd_property), 2, sgml, sgml).
+index((new_dtd), 2, sgml, sgml).
+index((free_dtd), 1, sgml, sgml).
+index((open_dtd), 3, sgml, sgml).
+index((new_sgml_parser), 2, sgml, sgml).
+index((free_sgml_parser), 1, sgml, sgml).
+index((set_sgml_parser), 2, sgml, sgml).
+index((get_sgml_parser), 2, sgml, sgml).
+index((sgml_parse), 2, sgml, sgml).
+index((sgml_register_catalog_file), 2, sgml, sgml).
+index((xml_quote_attribute), 3, sgml, sgml).
+index((xml_quote_cdata), 3, sgml, sgml).
+index((xml_quote_attribute), 2, sgml, sgml).
+index((xml_quote_cdata), 2, sgml, sgml).
+index((xml_name), 1, sgml, sgml).
+index((xml_name), 2, sgml, sgml).
+index((iri_xml_namespace), 2, sgml, sgml).
+index((iri_xml_namespace), 3, sgml, sgml).
+index((xml_is_dom), 1, sgml, sgml).
+index((html_write), 2, sgml_write, sgml_write).
+index((html_write), 3, sgml_write, sgml_write).
+index((sgml_write), 2, sgml_write, sgml_write).
+index((sgml_write), 3, sgml_write, sgml_write).
+index((xml_write), 2, sgml_write, sgml_write).
+index((xml_write), 3, sgml_write, sgml_write).
 index((sha_hash), 3, crypto_hash, sha).
 index((sha_new_ctx), 2, crypto_hash, sha).
 index((sha_hash_ctx), 4, crypto_hash, sha).
@@ -676,6 +752,7 @@ index((subsumes_chk), 2, terms, terms).
 index((cyclic_term), 1, terms, terms).
 index((acyclic_term), 1, terms, terms).
 index((term_subsumer), 3, terms, terms).
+index((term_factorized), 3, terms, terms).
 index((show_coverage), 1, prolog_cover, test_cover).
 index((covered_clauses), 4, prolog_cover, test_cover).
 index((make_tests), 3, test_wizard, test_wizard).
@@ -766,6 +843,13 @@ index((vm_list), 1, prolog_vm, vm).
 index((when), 2, when, when).
 index((www_open_url), 1, www_browser, www_browser).
 index((expand_url_path), 2, www_browser, www_browser).
+index((xpath), 3, xpath, xpath).
+index((xpath_chk), 3, xpath, xpath).
+index((xsdp_type), 1, xsdp_type, xsdp_types).
+index((xsdp_uri_type), 2, xsdp_type, xsdp_types).
+index((xsdp_numeric_uri), 2, xsdp_type, xsdp_types).
+index((xsdp_subtype_of), 2, xsdp_type, xsdp_types).
+index((xsdp_convert), 3, xsdp_type, xsdp_types).
 index((zopen), 3, zlib, zlib).
 index((gzopen), 3, zlib, zlib).
 index((gzopen), 4, zlib, zlib).
