@@ -38,6 +38,7 @@
 	    http_current_handler/3,	% ?Path, ?Pred
 	    http_location_by_id/2,	% +ID, -Location
 	    http_link_to_id/3,		% +ID, +Parameters, -HREF
+	    http_reload_with_parameters/3, % +Request, +Parameters, -HREF
 	    http_safe_file/2		% +Spec, +Options
 	  ]).
 :- use_module(library(option)).
@@ -425,6 +426,23 @@ http_link_to_id(HandleID, Parameters, HREF) :-
 	uri_data(search, Components, String),
 	uri_components(HREF, Components).
 
+%%	http_reload_with_parameters(+Request, +Parameters, -HREF) is det.
+%
+%	Create a request on the current handler with replaced search
+%	parameters.
+
+http_reload_with_parameters(Request, NewParams, HREF) :-
+	memberchk(path(Path), Request),
+	(   memberchk(search(Params), Request)
+	->  true
+	;   Params = []
+	),
+	merge_options(NewParams, Params, AllParams),
+	uri_query_components(Search, AllParams),
+	uri_data(path, Data, Path),
+	uri_data(search, Data, Search),
+	uri_components(HREF, Data).
+
 
 %	hook into html_write:attribute_value//1.
 
@@ -809,7 +827,8 @@ add_paths_tree([path(Path, Action, Options)|T], Tree0, Tree) :-
 
 plain_path(Path, Action, Options) :-
 	handler(Spec, Action, false, Options),
-	http_absolute_location(Spec, Path, []).
+	catch(http_absolute_location(Spec, Path, []), E,
+	      (print_message(error, E), fail)).
 
 
 %%	add_path_tree(+Path, +Action, +Options, +Tree0, -Tree) is det.
