@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2010, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -30,17 +31,23 @@
 */
 
 :- module(json_convert,
-	  [ prolog_to_json/2,		% +Term, -JSON object
-	    json_to_prolog/2,		% :JSON, -Term
+	  [ prolog_to_json/2,		% :Term, -JSON object
+	    json_to_prolog/2,		% +JSON, :Term
 	    (json_object)/1,		% +Definition
 	    op(1150, fx, (json_object))
 	  ]).
 :- use_module(library(error)).
 :- use_module(json).
 
-:- module_transparent
-	prolog_to_json/2,
-	json_to_prolog/2.
+:- meta_predicate
+	prolog_to_json(:, -),
+	json_to_prolog(+, :).
+
+:- public
+	clear_cache/0,
+	prolog_list_to_json/3,		% +ListIn, -ListOut, +Module
+	prolog_to_json/3,		% +In, -Out, +Module
+	prolog_bool_to_json/2.		% +In, -Boolean
 
 /** <module> Convert between JSON terms and Prolog application terms
 
@@ -108,7 +115,7 @@ simplifies debugging this rather complicated process.
 	though this may be unsafe.  If the Prolog term is known to be
 	non-ground and JSON @null is a sensible mapping, we can also use
 	this simple snipit to deal with that fact.
-	
+
 	==
 		term_variables(Term, Vars),
 		maplist(=(@null), Vars).
@@ -131,7 +138,7 @@ simplifies debugging this rather complicated process.
 %
 %	Declare a JSON object.  The declaration takes the same format as
 %	using in record/1 from library(record).  E.g.
-%	
+%
 %	==
 %	?- json_object
 %		point(x:int, y:int, z:int=0).
@@ -145,7 +152,7 @@ json_object(Declaration) :-
 %
 %	Compiles a :- json_object directive into Clauses. Clauses are of
 %	the form:
-%	
+%
 %	==
 %	json_object_to_pairs(Term, Module, Pairs) :-
 %		<type-checks on Term>,
@@ -210,14 +217,14 @@ record_to_json_clause(Constructor, Module, Types, Names, Extra) -->
 	},
 	[ (json_convert:(Head :- Body)) ].
 
-	      
+
 %%	type_checks(+Types, -VarsIn, -VarsOut, -Goal, +Module) is det.
 %
 %	Goal is a body-term  that  validates   Vars  satisfy  Types.  In
 %	addition to the types accepted by   must_be/2,  it accepts =any=
 %	and Name/Arity. The latter demands  a   json_object  term of the
 %	given Name and Arity.
-%	
+%
 %	@tbd	Compile list(Type) specification.  Currently Type is
 %		handled like =any=
 
@@ -242,7 +249,7 @@ type_check(Type, V, V, _, Goal) :-
 %	JSON is the JSON boolean for Prolog. It is a flexible the Prolog
 %	notation for thruth-value, accepting one of  =true=, =on= or =1=
 %	for @true and one of =false=, =fail=, =off= or =0= for @false.
-%	
+%
 %	@error	instantiation_error if Prolog is unbound.
 
 prolog_bool_to_json(Var, _) :-
@@ -256,7 +263,7 @@ prolog_bool_to_json(on, @(true)).
 prolog_bool_to_json(off, @(false)).
 prolog_bool_to_json(1, @(false)).
 prolog_bool_to_json(@(True), True) :-
-	prolog_bool_to_json(True, True). 
+	prolog_bool_to_json(True, True).
 
 
 %%	type_goal(+Type, +Var, -BodyTerm) is det.
@@ -264,9 +271,24 @@ prolog_bool_to_json(@(True), True) :-
 %	Inline type checking calls.
 
 type_goal(Type, Var, Body) :-
-	clause(error:has_type(Type, Var), Body), !.
+	clause(error:has_type(Type, Var), Body0),
+	primitive(Body0, Body), !.
 type_goal(Type, Var, is_of_type(Type, Var)).
 
+primitive((A0,B0), (A,B)) :- !,
+	primitive(A0, A),
+	primitive(B0, B).
+primitive((A0;B0), (A,B)) :- !,
+	primitive(A0, A),
+	primitive(B0, B).
+primitive((A0->B0), (A,B)) :- !,
+	primitive(A0, A),
+	primitive(B0, B).
+primitive(_:G, G) :- !,
+	predicate_property(system:G, built_in).
+primitive(G, G) :-
+	predicate_property(system:G, built_in).
+
 
 %%	clean_body(+BodyIn, -BodyOut) is det.
 %
@@ -315,7 +337,7 @@ current_clause(Constructor, Module, Types, Names, Extra) -->
 	  Head =.. [current_json_object, Term, Module, Fields]
 	},
 	[ json_convert:Head ].
-	
+
 extra_fields([], []).
 extra_fields([Name=Value|T0], [f(Name, oneof([Value]), Value)|T]) :-
 	extra_fields(T0, T).
@@ -336,7 +358,7 @@ defaults([Arg=Default|T0], [Default|TD], [Arg|TA]) :- !,
 	defaults(T0, TD, TA).
 defaults([Arg|T0], [_|TD], [Arg|TA]) :-
 	defaults(T0, TD, TA).
-	
+
 
 %%	types(+ArgsSpecs, -Defaults, -Args)
 %
@@ -363,13 +385,12 @@ types([Name|T0], [Name|TN], [any|TT]) :-
 %	=boolean=, commonly used thruth-values in   Prolog are converted
 %	to JSON booleans. Boolean  translation   accepts  one of =true=,
 %	=on=, =1=, @true, =false=, =fail=, =off= or =0=, @false.
-%	
+%
 %	@error	type_error(json_term, X)
 %	@error	instantiation_error
 
-prolog_to_json(Term, JSON) :-
-	strip_module(Term, M, T),
-	prolog_to_json(T, JSON, M).
+prolog_to_json(Module:Term, JSON) :-
+	prolog_to_json(Term, JSON, Module).
 
 prolog_to_json(Var, _, _) :-
 	var(Var), !,
@@ -425,14 +446,13 @@ clear_cache :-
 %	assumption that, although the order of   fields in JSON terms is
 %	irrelevant and can therefore vary  a lot, practical applications
 %	will normally generate the JSON objects in a consistent order.
-%	
+%
 %	If a field in a json_object is declared of type =boolean=, @true
 %	and @false are  translated  to  =true=   or  =false=,  the  most
 %	commonly used Prolog representation for truth-values.
 
-json_to_prolog(JSON, Term) :-
-	strip_module(Term, M, T),
-	json_to_prolog(JSON, T, M).
+json_to_prolog(JSON, Module:Term) :-
+	json_to_prolog(JSON, Term, Module).
 
 json_to_prolog(json(Pairs), Term, Module) :- !,
 	(   pairs_to_term(Pairs, Term, Module)
@@ -469,7 +489,7 @@ json_object_to_prolog(json(Pairs), Term, Module) :-
 %
 %	Convert a Name=Value set into a Prolog application term based on
 %	json_object/1 declarations.
-%	
+%
 %	@tbd	Ignore extra pairs if term is partially given?
 
 pairs_to_term(Pairs, Term, Module) :-
@@ -485,7 +505,7 @@ pairs_to_term(Pairs, Term, Module) :-
 	    asserta(created_rules_for_pairs(M, PairArgs)),
 	    json_to_prolog_rule(M, Pairs, Term), !
 	).
-	
+
 pairs_args([], []).
 pairs_args([Name=_Value|T0], [Name=_|T]) :-
 	pairs_args(T0, T).
@@ -494,7 +514,7 @@ pairs_args([Name=_Value|T0], [Name=_|T]) :-
 %
 %	Create a new rule for dealing with Pairs, a Name=Value list of a
 %	particular order.  Here is an example rule:
-%	
+%
 %	==
 %	json_to_prolog_rule([x=X, y=Y], point(X,Y)) :-
 %		integer(X),
@@ -515,6 +535,9 @@ match_field(any, JSON, Prolog, M, json_to_prolog(JSON,Prolog,M)) :- !.
 match_field(F/A, JSON, Prolog, M, json_to_prolog(JSON,Prolog,M)) :- !,
 	functor(Prolog, F, A).
 match_field(boolean, JSON, Prolog, _, json_bool_to_prolog(JSON, Prolog)) :- !.
+match_field(list(Type), JSON, Prolog, M, json_list_to_prolog(JSON, Prolog, M)) :-
+	current_json_object(Term, M, _Fields),
+	functor(Term, Type, _), !.
 match_field(Type, Var, Var, _, Goal) :-
 	type_goal(Type, Var, Goal).
 
@@ -526,9 +549,9 @@ json_bool_to_prolog(@(True), True).
 		 *******************************/
 
 :- multifile
-	user:term_expansion/2.
+	system:term_expansion/2.
 :- dynamic
-	user:term_expansion/2.
+	system:term_expansion/2.
 
-user:term_expansion((:- json_object(Spec)), Clauses) :-
+system:term_expansion((:- json_object(Spec)), Clauses) :-
 	compile_json_objects(Spec, Clauses).
