@@ -75,6 +75,7 @@
 :- use_module(library(readutil)).
 :- use_module(library(http/html_write)).
 :- use_module(library(http/http_dispatch)).
+:- use_module(library(http/http_wrapper)).
 :- use_module(library(http/http_path)).
 :- use_module(library(http/html_head)).
 :- use_module(library(doc_http)).
@@ -288,7 +289,8 @@ private(Module:PI, _Options) :-
 %
 %	True if Obj is a multifile predicate.
 
-multifile(Module:PI, _Options) :-
+multifile(Obj, _Options) :-
+	strip_module(user:Obj, Module, PI),
 	pi_to_head(PI, Head),
 	predicate_property(Module:Head, multifile).
 
@@ -383,8 +385,7 @@ edit_button(File, Options) -->
 	}, !,
 	html(a([ onClick('HTTPrequest(\'' +
 			 location_by_id(pldoc_edit) + [file(File)] +
-			 '\')'),
-		 onMouseOver('window.status=\'Edit file\'; return true;')
+			 '\')')
 	       ],
 	       img([ class(action),
 		     alt(edit),
@@ -1475,7 +1476,7 @@ in_file(Module, Head, File) :-
 
 %%	relative_file_name(+Path:atom, +RelTo:atom, -RelPath:atom) is det.
 %
-%	Create a relative path from an absolute one.
+%	True when RelPath is a relative path to Path, relative to RelTo.
 %
 %	@tbd	move to library?
 
@@ -1530,6 +1531,17 @@ file(File, Options) -->
 	  merge_options(Options, GenOptions, FinalOptions)
 	},
 	link_file(File, FinalOptions), !.
+file(File, Options) -->
+	{ option(edit_handler(Handler), Options),
+	  http_current_request(Request),
+	  memberchk(path(Path), Request),
+	  absolute_file_name(File, Location,
+			     [ relative_to(Path)
+			     ]),
+	  http_link_to_id(Handler, [location(Location)], HREF),
+	  format(atom(Title), 'Click to create ~w', [File])
+	},
+	html(a([href(HREF), class(nofile), title(Title)], File)).
 file(File, _) -->
 	html(code(class(nofile), File)).
 
@@ -1575,7 +1587,7 @@ file_href_real(File, HREF, Options) :-
 	),
 	file_href(LinkFile, HREF).
 file_href_real(File, HREF, _) :-
-	http:location(Alias, _, _),
+	directory_alias(Alias),
 	Term =.. [Alias,File],
 	absolute_file_name(Term, _,
 			   [ access(read),
@@ -1583,6 +1595,10 @@ file_href_real(File, HREF, _) :-
 			   ]), !,
 	http_absolute_location(Term, HREF, []).
 
+directory_alias(icons).
+directory_alias(css).
+
+
 %%	file_href(+FilePath, -HREF) is det.
 %
 %	Create a relative URL from  the   current  location to the given
@@ -1699,18 +1715,19 @@ generalise_spec(Name//Arity, _M:Name//Arity).
 %
 %	Write HTML for the File containing wiki data.
 
-doc_for_wiki_file(FileSpec, _Options) :-
+doc_for_wiki_file(FileSpec, Options) :-
 	absolute_file_name(FileSpec, File,
 			   [ access(read)
 			   ]),
 	read_file_to_codes(File, String, []),
 	b_setval(pldoc_file, File),
-	call_cleanup(reply_wiki_page(File, String),
+	call_cleanup(reply_wiki_page(File, String, Options),
 		     nb_delete(pldoc_file)).
 
-reply_wiki_page(File, String) :-
-	wiki_codes_to_dom(String, [], DOM),
-	title(DOM, File, Title),
+reply_wiki_page(File, String, Options) :-
+	wiki_codes_to_dom(String, [], DOM0),
+	title(DOM0, File, Title),
+	insert_edit_button(DOM0, File, DOM, Options),
 	reply_html_page(pldoc(wiki),
 			title(Title),
 			[ \html_requires(pldoc)
@@ -1722,6 +1739,21 @@ title(DOM, _, Title) :-
 title(_, File, Title) :-
 	file_base_name(File, Title).
 
+insert_edit_button(DOM, _, DOM, Options) :-
+	option(edit(false), Options, false), !.
+insert_edit_button([h1(Attrs,Title)|DOM], File,
+		   [h1(Attrs,[ span(style('float:right'),
+				   \edit_button(File, [edit(true)]))
+			     | Title
+			     ])|DOM], _) :- !.
+insert_edit_button(DOM, File,
+		   [ h1(class(wiki),
+			[ span(style('float:right'),
+			       \edit_button(File, [edit(true)]))
+			])
+		   | DOM
+		   ], _).
+
 
 		 /*******************************
 		 *	      ANCHORS		*
