@@ -36,16 +36,18 @@
 	    prolog_canonical_source/2,	% +Spec, -Id
 
 	    file_name_on_path/2,	% +File, -PathSpec
-	    file_alias_path/2		% ?Alias, ?Dir
+	    file_alias_path/2,		% ?Alias, ?Dir
+	    path_segments_atom/2	% ?Segments, ?Atom
 	  ]).
 :- use_module(operators).
 :- use_module(debug).
+:- use_module(option).
 
 /** <module> Examine Prolog source-files
 
-The modile prolog_source.pl provides predicates to  open, close and read
-terms from Prolog source-files. This may  seem   easy,  but  there are a
-couple of problems that must be taken care of.
+This module provides predicates  to  open,   close  and  read terms from
+Prolog source-files. This may seem  easy,  but   there  are  a couple of
+problems that must be taken care of.
 
 	* Source files may start with #!, supporting PrologScript
 	* Embedded operators declarations must be taken into account
@@ -63,11 +65,13 @@ users of the library are:
 */
 
 :- thread_local
-	open_source/2.		% Stream, State
+	open_source/2,		% Stream, State
+	mode/2.			% Stream, Data
 
 :- multifile
 	requires_library/2,
 	prolog:xref_source_identifier/2,	% +Source, -Id
+	prolog:xref_source_time/2,		% +Source, -Modified
 	prolog:xref_open_source/2.		% +SourceId, -Stream
 
 
@@ -78,31 +82,89 @@ users of the library are:
 %%	prolog_read_source_term(+In, -Term, -Expanded, +Options) is det.
 %
 %	Read a term from a Prolog source-file.  Options is a option list
-%	that is forwarded to read_term/3.
+%	that is forwarded to read_term/3.  In addition, it accepts:
+%
+%	    * process_comment(+Boolean)
+%	    If =true=, process structured comments for PlDoc
 %
 %	@param Term	Term read
 %	@param Expanded	Result of term-expansion on the term
 
 prolog_read_source_term(In, Term, Expanded, Options) :-
+	read_source_term(In, Term, Options),
+	expand(Term, In, Expanded),
+	update_state(Expanded).
+
+:- multifile
+	prolog:comment_hook/3.
+
+read_source_term(In, Term, Options) :-
+	'$get_predicate_attribute'(prolog:comment_hook(_,_,_),
+				   number_of_clauses, N),
+	N > 0,
+	option(process_comment(true), Options), !,
+	select_option(term_position(TermPos), Options, Options1, _),
+	'$set_source_module'(SM, SM),
+	read_term(In, Term,
+		  [ comments(Comments),
+		    module(SM),
+		    term_position(TermPos)
+		  | Options1
+		  ]),
+	(   catch(prolog:comment_hook(Comments, TermPos, Term), E,
+		  print_message(error, E))
+	->  true
+	;   true
+	).
+read_source_term(In, Term, Options) :-
 	'$set_source_module'(SM, SM),
 	read_term(In, Term,
 		  [ module(SM)
 		  | Options
-		  ]),
-	expand(Term, Expanded),
-	update_state(Expanded).
+		  ]).
 
-expand(Var, Var) :-
+
+expand(Var, _, Var) :-
 	var(Var), !.
-expand(Term, _) :-
+expand(Term, _, Term) :-
+	no_expand(Term), !.
+expand(Term, _, _) :-
 	requires_library(Term, Lib),
 	ensure_loaded(user:Lib),
 	fail.
-expand('$:-'(X), '$:-'(X)) :- !,	% boot module
-	style_check(+dollar).
-expand(Term, Expanded) :-
+expand(Term, In, Term) :-
+	chr_expandable(Term, In), !.
+expand(Term, _, Expanded) :-
 	expand_term(Term, Expanded).
 
+no_expand((:- if(_))).
+no_expand((:- elif(_))).
+no_expand((:- else)).
+no_expand((:- endif)).
+no_expand((:- require(_))).
+
+chr_expandable((:- chr_constraint(_)), In) :-
+	add_mode(In, chr).
+chr_expandable((handler(_)), In) :-
+	mode(In, chr).
+chr_expandable((rules(_)), In) :-
+	mode(In, chr).
+chr_expandable(<=>(_, _), In) :-
+	mode(In, chr).
+chr_expandable(@(_, _), In) :-
+	mode(In, chr).
+chr_expandable(==>(_, _), In) :-
+	mode(In, chr).
+chr_expandable(pragma(_, _), In) :-
+	mode(In, chr).
+chr_expandable(option(_, _), In) :-
+	mode(In, chr).
+
+add_mode(Stream, Mode) :-
+	mode(Stream, Mode), !.
+add_mode(Stream, Mode) :-
+	asserta(mode(Stream, Mode)).
+
 %%	requires_library(+Term, -Library)
 %
 %	known expansion hooks.  May be expanded as multifile predicate.
@@ -196,10 +258,17 @@ prolog_open_source(Src, Fd) :-
 %%	prolog_close_source(+In:stream) is det.
 %
 %	Close  a  stream  opened  using  prolog_open_source/2.  Restores
-%	operator and style options.
+%	operator and style options. If the stream   has not been read to
+%	the end, we call expand_term(end_of_file,  _) to allow expansion
+%	modules to clean-up.
 
 prolog_close_source(In) :-
+	(   at_end_of_stream(In)
+	->  true
+	;   ignore(catch(expand(end_of_file, In, _), _, true))
+	),
 	pop_operators,
+	retractall(mode(In, _)),
 	(   retract(open_source(In, state(LexState, SM)))
 	->  '$restore_lex_state'(LexState),
 	    '$set_source_module'(_, SM)
@@ -297,3 +366,50 @@ ensure_slash(Dir, Dir) :-
 	sub_atom(Dir, _, _, 0, /), !.
 ensure_slash(Dir0, Dir) :-
 	atom_concat(Dir0, /, Dir).
+
+
+%%	path_segments_atom(+Segments, -Atom) is det.
+%%	path_segments_atom(-Segments, +Atom) is det.
+%
+%	Translate between a path  represented  as   a/b/c  and  an  atom
+%	representing the same path. For example:
+%
+%	  ==
+%	  ?- path_segments_atom(a/b/c, X).
+%	  X = 'a/b/c'.
+%	  ?- path_segments_atom(S, 'a/b/c'), display(S).
+%	  /(/(a,b),c)
+%	  S = a/b/c.
+%	  ==
+%
+%	This predicate is part of  the   Prolog  source  library because
+%	SWI-Prolog  allows  writing  paths   as    /-nested   terms  and
+%	source-code analysis programs often need this.
+
+path_segments_atom(Segments, Atom) :-
+	var(Atom), !,
+	(   atomic(Segments)
+	->  Atom = Segments
+	;   segments_to_list(Segments, List, [])
+	->  atomic_list_concat(List, /, Atom)
+	;   throw(error(type_error(file_path, Segments), _))
+	).
+path_segments_atom(Segments, Atom) :-
+	atomic_list_concat(List, /, Atom),
+	parts_to_path(List, Segments).
+
+segments_to_list(Var, _, _) :-
+	var(Var), !, fail.
+segments_to_list(A/B, H, T) :-
+	segments_to_list(A, H, T0),
+	segments_to_list(B, T0, T).
+segments_to_list(A, [A|T], T) :-
+	atomic(A).
+
+parts_to_path([One], One) :- !.
+parts_to_path(List, More/T) :-
+	(   append(H, [T], List)
+	->  parts_to_path(H, More)
+	).
+
+
