@@ -38,7 +38,9 @@
 	    send_if_tracer/1,		% :Goal
 	    get_tracer/2,		% :Goal, -Result
 	    in_debug_thread/2,		% +ObjOrThread, :Goal
-	    prolog_frame_attribute/4	% +GUI, +Frame, +Attr, -Value
+	    thread_debug_queue/2,	% +Thread, -Queue
+	    prolog_frame_attribute/4,	% +GUI, +Frame, +Attr, -Value
+	    prolog_choice_attribute/4	% +GUI, +Choice, +Attr, -Value
 	  ]).
 :- use_module(library(pce)).
 :- use_module(library(lists)).
@@ -61,6 +63,10 @@
 	user:prolog_event_hook/1,
 	user:message_hook/3.
 
+:- meta_predicate
+	in_debug_thread(+, 0),
+	send_pce(0).
+
 register_directories :-
 	(   member(SpyBase, ['icons/nospy', library('trace/icons/nospy')]),
 	    absolute_file_name(SpyBase,
@@ -75,8 +81,8 @@ register_directories :-
 			       HlpFile)
 	->  pce_help_file(pltracer, HlpFile)
 	).
-	    
-:- initialization register_directories.
+
+:- register_directories.
 
 version('2.0').
 
@@ -113,7 +119,7 @@ prolog_tracer(Thread, Ref, Create) :-
 %%	break_level(-Level)
 %
 %	Current break-level.
-%	
+%
 %	@bug	Breaks only work in the main thread. We need to change I/O
 %		handling and where the break-level is stored to fix
 %		this.
@@ -129,12 +135,13 @@ break_level(1).
 %%	get_tracer(+Thread, +Term, -Reply) is semidet.
 %
 %	Send messages to the XPCE tracer window.
-%	
+%
 %	@param Thread: calling thread.
 
 send_tracer(Term) :-
 	thread_self(Thread),
 	send_tracer(Thread, Term).
+
 send_tracer(GUI, Term) :-
 	object(GUI), !,
 	send_pce(send(GUI, Term)).
@@ -154,6 +161,7 @@ send_if_tracer(Thread, Term) :-
 get_tracer(Term, Result) :-
 	thread_self(Thread),
 	get_tracer(Thread, Term, Result).
+
 get_tracer(GUI, Term, Result) :-
 	object(GUI), !,
 	get(GUI, Term, Result).
@@ -166,27 +174,46 @@ get_tracer(Thread, Term, Result) :-
 		 *    THREAD SYNCHRONISATION	*
 		 *******************************/
 
-:- meta_predicate
-	in_debug_thread(+, :),
-	send_pce(:).
+%%	thread_debug_queue(+Thread, -Queue) is det.
+%
+%	Queue is the debugging queue for Thread.  We do not use the main
+%	queue to avoid interference with user-messages.
+
+:- dynamic
+	thread_debug_queue_store/2.
+
+thread_debug_queue(Thread, Queue) :-
+	(   thread_debug_queue_store(Thread, Q)
+	->  Queue = Q
+	;   message_queue_create(Q),
+	    assert(thread_debug_queue_store(Thread, Q)),
+	    Queue = Q
+	).
+
+:- multifile
+	user:prolog_event_hook/1.
+
+user:prolog_event_hook(thread_finished(TID)) :-
+	forall(retract(thread_debug_queue_store(TID, Queue)),
+	       message_queue_destroy(Queue)),
+	fail.				% allow other hooks
 
 %%	send_pce(:Goal)
 %
-%	Run Goal in XPCE (main)  thread.   Wait  for  completion. In the
-%	meanwhile, allow the XPCE thread to call in_debug_thread/1.
-%	
-%	@bug	XPCE thread does not _need_ to be =main=.
+%	Run Goal in XPCE thread. Wait  for completion. In the meanwhile,
+%	allow the XPCE thread to call in_debug_thread/1.
 
 send_pce(Goal) :-
-	thread_self(main), !,
+	thread_self(Me),
+	pce_thread(Me), !,
 	Goal.
 send_pce(Goal) :-
 	thread_self(Self),
-	strip_module(Goal, M, G),
-	term_variables(G, GVars),
-	in_pce_thread(run_pce(M:G, GVars, Self)),
+	term_variables(Goal, GVars),
+	in_pce_thread(run_pce(Goal, GVars, Self)),
+	thread_debug_queue(Self, Queue),
 	repeat,
-	thread_get_message('$trace'(Result)),
+	thread_get_message(Queue, '$trace'(Result)),
 	debug(' ---> send_pce: result = ~p~n', [Result]),
 	(   Result = error(E)
 	->  throw(E)
@@ -209,7 +236,8 @@ run_pce(Goal, Vars, Caller) :-
 	;   Result = false
 	),
 	debug('Ok, returning ~p~n', [Result]),
-	thread_send_message(Caller, '$trace'(Result)).
+	thread_debug_queue(Caller, Queue),
+	thread_send_message(Queue, '$trace'(Result)).
 
 %%	in_debug_thread(+Thread, :Goal) is semidet.
 %%	in_debug_thread(+Object, :Goal) is semidet.
@@ -227,12 +255,13 @@ in_debug_thread(Thread, Goal) :-
 	thread_self(Thread), !,
 	Goal, !.
 in_debug_thread(Thread, Goal) :-
-	strip_module(Goal, M, G),
 	thread_self(Self),
-	debug('Call [Thread ~p] ~p~n', [Thread, M:G]),
-	term_variables(G, GVars),
-	thread_send_message(Thread, '$trace'(call(M:G, GVars, Self))),
-	thread_get_message('$trace'(Result)),
+	debug('Call [Thread ~p] ~p~n', [Thread, Goal]),
+	term_variables(Goal, GVars),
+	thread_debug_queue(Thread, Queue),
+	thread_send_message(Queue, '$trace'(call(Goal, GVars, Self))),
+	thread_debug_queue(Self, MyQueue),
+	thread_get_message(MyQueue, '$trace'(Result)),
 	debug(' ---> in_debug_thread: result = ~p~n', [Result]),
 	(   Result = error(E)
 	->  throw(E)
@@ -251,6 +280,11 @@ prolog_frame_attribute(Thread, Frame, Attribute, Value) :-
 	in_debug_thread(Thread,
 			prolog_frame_attribute(Frame, Attribute, Value)).
 
+prolog_choice_attribute(GUI, Choice, Attribute, Value) :-
+	in_debug_thread(GUI,
+			prolog_choice_attribute(Choice, Attribute, Value)).
+
+
 
 		 /*******************************
 		 *     DEBUGGER APPLICATION	*
@@ -278,14 +312,18 @@ initialise(App) :->
 variable(source,	any,		both, "Source view").
 variable(break_level,	int,		get,  "Break-level I'm associated to").
 variable(thread,	'int|name*',	get,  "Associated thread").
+variable(trap_frame,    int*,   	get,  "Last trapped frame").
+variable(trap_port,     name*,   	get,  "Last trapped port").
 variable(current_frame, int*,   	both, "The most recent frame").
 variable(current_break,	tuple*,		both, "tuple(ClauseRef, PC)").
 variable(quitted,	bool := @off,   both, "Asked to quit").
 variable(mode,		name := created,both, "Current mode").
 
-initialise(F, Level:int, Thread0:'int|name') :->
-	assertion(thread_self(main)),
-	default(Thread0, main, Thread),
+running_in_pce_thread :-
+	pce_thread(Pce), thread_self(Pce).
+
+initialise(F, Level:int, Thread:'int|name') :->
+	assertion(running_in_pce_thread),
 	send(F, slot, break_level, Level),
 	send(F, slot, thread, Thread),
 	send_super(F, initialise, 'SWI-Prolog debugger', application := @prolog_gui),
@@ -314,7 +352,7 @@ initialise(F, Level:int, Thread0:'int|name') :->
 	send(S, name, stack),
 	ignore(send(F, frame_finished, 0)),	% FR_WATCHED issue
 	asserta(gui(Thread, Level, F)).
- 
+
 unlink(F) :->
 	retractall(gui(_, _, F)),
 	clear_clause_info_cache,	% safety first
@@ -325,6 +363,7 @@ quit(F) :->
 	(   (   get(F, mode, thread_finished)
 	    ;   get(F, mode, query_finished)
 	    ;   get(F, mode, aborted)
+	    ;	get(F, mode, replied)
 	    )
 	->  send(F, destroy)
 	;   get(F, tracer_quitted, Action),
@@ -381,7 +420,10 @@ fill_menu_bar(F) :->
 			      message(F, exceptions),
 			      end_group := @on),
 		    menu_item(toggle_edit_mode,
-			      message(F, edit))
+			      message(F, edit),
+			      end_group := @on),
+		    menu_item(copy_goal,
+			      message(F, copy_goal))
 		  ]),
 	send_list(View, append,
 		  [ menu_item(threads,
@@ -421,9 +463,18 @@ help(_) :->
         "Show window with help-text"::
         send(@helper, give_help, pltracer, main).
 
-show_frame(GUI, Frame:int, PC:'int|name') :->
+show_frame(GUI, Frame:int, PC:prolog) :->
 	"Show the variables of this frame"::
-	prolog_show_frame(Frame, [gui(GUI), pc(PC), source, bindings]).
+	(   get(GUI, trap_frame, Frame)		% the initial trapped port
+	->  get(GUI, trap_port, Style)
+	;   PC = choice(_)			% A choice-point
+	->  Style = choice
+	;   Style = frame			% Somewhere up the stack
+	),
+	prolog_show_frame(Frame, [ gui(GUI), pc(PC),
+				   source, bindings,
+				   style(Style)
+				 ]).
 
 show_stack(GUI, CallFrames:prolog, ChoiceFrames:prolog) :->
 	"Show the stack and choicepoints"::
@@ -435,6 +486,12 @@ show_threads(_GUI) :->
 	"Open Thread monitor"::
 	prolog_ide(thread_monitor).
 
+trapped_location(GUI, StartFrame:int, Frame:int, Port:name) :->
+	"The last trapped location"::
+	send(GUI, slot, trap_frame, Frame),
+	send(GUI, slot, trap_port, Port),
+	send(GUI, current_frame, StartFrame).
+
 
 		 /*******************************
 		 *	      EVENT		*
@@ -479,14 +536,16 @@ return(Frame, Result:any) :->
 	(   get(Frame, mode, wait_user)
 	->  get(Frame, thread, Thread),
 	    send(Frame, mode, replied),
-	    (   Thread == main
-	    ->  send_super(Frame, return, Result)
+	    (	pce_thread(Thread)
+	    ->	send_super(Frame, return, Result)
 	    ;   (   get(Frame, quitted, @on)
 		->  send(Frame, destroy)
 		;   true
 		),
-		debug(' ---> frame: result = ~p~n', [Result]),
-		thread_send_message(Thread, '$trace'(action(Result)))
+		debug(' ---> frame for thread = ~p: result = ~p~n',
+		      [Thread, Result]),
+		thread_debug_queue(Thread, Queue),
+		thread_send_message(Queue, '$trace'(action(Result)))
 	    )
 	;   get(Frame, quitted, @on)
 	->  send(Frame, destroy)
@@ -593,7 +652,7 @@ nostop_or_spy(F) :->
 	;   send(F, report, warning,
 		 'No selected break or current spy-point')
 	).
-	    
+
 browse(_F) :->
 	"Provides overview for edit/spy/break"::
 	prolog_ide(open_navigator).
@@ -618,10 +677,45 @@ details(F) :->
 	get(F, member, bindings, Bindings),
 	send(Bindings, details).
 
+nodebug(F) :->
+	"User hit n(odebug)"::
+	(   setting(auto_close, true)
+	->  send(F, quitted, @on)
+	;   true
+	),
+	send(F, return, nodebug).
+
+abort(F) :->
+	"User hit a(bort)"::
+	(   setting(auto_close, true)
+	->  send(F, quitted, @on)
+	;   true
+	),
+	send(F, return, abort).
+
 query(_F) :->
 	"Enter and run a query"::
 	prolog_ide(open_query_window).
 
+copy_goal(F) :->
+	"Copy the current goal into the copy-buffer"::
+	get(F, selected_frame, Frame),
+	(   Frame \== @nil
+	->  true
+	;   send(F, report, warning, 'No current frame'),
+	    fail
+	),
+	prolog_frame_attribute(F, Frame, goal, Goal),
+	prolog_frame_attribute(F, Frame, predicate_indicator, PI),
+	(   numbervars(Goal, 0, _, [attvar(skip)]),
+	    format(string(Text), '~q', [Goal]),
+	    send(@display, copy, Text),
+	    fail
+	;   true
+	),
+	format(atom(PIA), '~q', [PI]),
+	send(F, report, inform, 'Copied goal (%s) to clipboard', PIA).
+
 :- pce_group(delegate).
 
 file(F, File:'name|emacs_buffer*') :->
@@ -701,9 +795,9 @@ button(finish,	       "f",   'finish.xpm',	     'Finish selected goal').
 button(gap,	       -,     -,		     -).
 button(retry,	       "r",   'retry.xpm',	     'Retry selected goal').
 button(gap,	       -,     -,		     -).
-button(nodebug,	       "n",   'nodebug.xpm',	     'Continue without debugging').
+button(+nodebug,       "n",   'nodebug.xpm',	     'Continue without debugging').
 button(+query,	       "b",   'break.xpm',	     'Enter a query').
-button(abort,	       "a",   'abort.xpm',	     'Abort to the Prolog toplevel').
+button(+abort,	       "a",   'abort.xpm',	     'Abort to the Prolog toplevel').
 button(fail,	       "F",   'fail.xpm',	     'Force query to fail').
 button(gap,	       -,     -,		     -).
 button(+up,	       "u",   'up.xpm',		     'Select parent frame').
@@ -722,8 +816,8 @@ button(+edit,	       "e",   'edit.xpm',	     'Toggle read-only/edit-mode').
 
 tag_balloon(Balloon0, Keys, Balloon) :-
 	maplist(key_name, Keys, Names),
-	concat_atom(Names, ', ', Tag),
-	concat_atom([Balloon0, ' (', Tag, ')'], Balloon).
+	atomic_list_concat(Names, ', ', Tag),
+	atomic_list_concat([Balloon0, ' (', Tag, ')'], Balloon).
 
 key_name(10, return) :- !.
 key_name(32,  space) :- !.
@@ -731,7 +825,7 @@ key_name(C, A) :-
 	char_code(A, C).
 
 initialise(D) :->
-	send_super(D, initialise),	
+	send_super(D, initialise),
 	send(D, pen, 0),
 	send(D, gap, size(0,0)),
 	get(D, frame, Frame),
@@ -757,7 +851,7 @@ make_message(Action,  D, message(D, return, Action)).
 
 typed(D, Id:event_id, Delegate:[bool]) :->
 	"Handle typing"::
-	(   get(D, find, @default, 
+	(   get(D, find, @default,
 		and(message(@arg1, has_get_method, keys),
 		    message(@arg1?keys, member, Id)),
 		Button)
@@ -802,6 +896,8 @@ variable(prolog_frame, int*, both, "Frame who's variables we are showing").
 
 :- pce_global(@prolog_binding_recogniser,
 	      make_prolog_binding_recogniser).
+:- pce_global(@prolog_binding_popup,
+	      make_prolog_binding_popup).
 
 make_prolog_binding_recogniser(G) :-
 	new(View, @event?window),
@@ -810,8 +906,18 @@ make_prolog_binding_recogniser(G) :-
 			      message(View, on_click, Index))),
 	new(C2, click_gesture(left, '', double,
 			      message(View, details))),
-	new(G, handler_group(C1, C2)).
+	new(C3, popup_gesture(@prolog_binding_popup)),
+	send(@prolog_binding_popup, update_message,
+	     message(View, on_click, Index)),
+	new(G, handler_group(C1, C2, C3)).
 
+make_prolog_binding_popup(P) :-
+	new(P, popup),
+	send_list(P, append,
+		  [ menu_item(details, message(@arg1?window, details)),
+		    menu_item(copy,    message(@arg1?window, details,
+					       @default, copy))
+		  ]).
 
 initialise(B) :->
 	send_super(B, initialise),
@@ -823,6 +929,7 @@ initialise(B) :->
 	send(B?image, tab_stops, vector(Tab)),
 	send(B?image, recogniser, @prolog_binding_recogniser),
 	send(B, editable, @off),
+	send(B, style, constraint, style(colour := blue)),
 	send(B?text_cursor, displayed, @off),
 	send(B, ver_stretch, 0).
 
@@ -833,7 +940,7 @@ clear(B, Content:[bool]) :->
 	;   send_super(B, clear)
 	).
 
-details(B, Fragment:[prolog_frame_var_fragment]) :->
+details(B, Fragment:[prolog_frame_var_fragment], Action:[{view,copy}]) :->
 	"View details of the binding"::
 	get(B, prolog_frame, Frame),
 	(   Frame \== @nil
@@ -861,11 +968,20 @@ details(B, Fragment:[prolog_frame_var_fragment]) :->
 	),
 	format(string(Label), '~w ~w of frame at level ~d running ~w',
 		[ VarType, VarName, Level, PredName ]),
-	view_term(Value,
-		  [ comment(Label),
-		    source_object(Frag),
-		    expose(true)
-		  ]).
+	debug('Action ~w on ~w~n', [Action, Value]),
+	(   Action == copy
+	->  (   numbervars(Value, 0, _, [attvar(skip)]),
+	        format(string(Text), '~q', [Value]),
+		send(@display, copy, Text),
+		fail
+	    ;   send(B, report, status, Label)
+	    )
+	;   view_term(Value,
+		      [ comment(Label),
+			source_object(Frag),
+			expose(true)
+		      ])
+	).
 
 on_click(B, Index:int) :->
 	"Select fragment clicked"::
@@ -878,30 +994,55 @@ on_click(B, Index:int) :->
 
 bindings(B, Bindings:prolog) :->
 	"Display complete list of bindings"::
+	(   term_attvars(Bindings, [])
+	->  Plain = Bindings,
+	    Constraints = []
+	;   copy_term(Bindings, Plain, Constraints)
+	),
 	send(B, background, white),
 	pce_open(B, write, Fd),
-	forall(member(Vars=Value, Bindings),
-	       send(B, append_binding, Vars, value(Value), Fd)),
+	(   bind_vars(Plain),
+	    forall(member(Vars=Value, Plain),
+		   send(B, append_binding, Vars, value(Value), Fd)),
+	    forall(member(C, Constraints),
+		   send(B, append_constraint, C, Fd)),
+	    fail
+	;   true
+	),
 	close(Fd),
 	send(B, caret, 0).
 
-append_binding(B, Names:prolog, ValueTerm:prolog, Fd:prolog) :->
+bind_vars([]).
+bind_vars([Vars=Value|T]) :-
+	(   var(Value)
+	->  Vars = [Name:_|_],
+	    Value = '$VAR'(Name)
+	;   true
+	),
+	bind_vars(T).
+
+
+append_binding(B, Names0:prolog, ValueTerm:prolog, Fd:prolog) :->
 	"Add a binding to the browser"::
-	ValueTerm = value(Value),	% protect :=, ?, etc.
-	(   var(Value),
+	ValueTerm = value(Value0),	% protect :=, ?, etc.
+	(   Value0 = '$VAR'(_), Names0 = [_],
 	    setting(show_unbound, false)
 	->  true
-	;   get(B, text_buffer, TB),
+	;   (   Value0 = '$VAR'(_), Names0 = [_,_|_]
+	    ->	append(Names, [VarN:_], Names0),
+		Value = '$VAR'(VarN)
+	    ;	Names = Names0,
+		Value = Value0
+	    ),
+	    get(B, text_buffer, TB),
 	    get(TB, size, S0),
 	    (   Names = VarName:ArgN
 	    ->	format(Fd, '~w', [VarName])
 	    ;	Names = [VarName:ArgN|_],
 	        write_varnames(Fd, Names)
 	    ),
-	    setting(attributes, Atts),
-	    current_prolog_flag(toplevel_print_options, Options0),
-	    delete(Options0, attributes(_), Options1),
-	    format(Fd, '\t= ~W~n', [Value, [attributes(Atts)|Options1]]),
+	    current_prolog_flag(toplevel_print_options, Options),
+	    format(Fd, '\t= ~W~n', [Value, Options]),
 	    flush_output(Fd),
 	    get(TB, size, S1),
 	    new(_, prolog_frame_var_fragment(TB, S0, S1, VarName, ArgN))
@@ -913,6 +1054,16 @@ write_varnames(Fd, [N:_|T]) :-
 	format(Fd, '~w = ', N),
 	write_varnames(Fd, T).
 
+append_constraint(B, Constraint:prolog, Fd:prolog) :->
+	"Display current constraints"::
+	get(B, text_buffer, TB),
+	current_prolog_flag(toplevel_print_options, Options),
+	get(TB, size, S0),
+	format(Fd, '(constraint)\t~W~n', [Constraint, Options]),
+	flush_output(Fd),
+	get(TB, size, S1),
+	new(_, prolog_frame_constraint_fragment(TB, S0, S1)).
+
 :- pce_end_class(prolog_bindings_view).
 
 
@@ -938,10 +1089,21 @@ value(F, Value:prolog) :<-
 	get(View, prolog_frame, Frame), Frame \== @nil,
 	get(F, argn, ArgN),
 	prolog_frame_attribute(F, Frame, argument(ArgN), Value).
-	
+
 :- pce_end_class(prolog_frame_var_fragment).
 
 
+:- pce_begin_class(prolog_frame_constraint_fragment, fragment,
+		   "Represent a contraint on a frame").
+
+initialise(F, TB:text_buffer, From:int, To:int) :->
+	Len is To-From,
+	send_super(F, initialise, TB, From, Len, constraint).
+
+:- pce_end_class(prolog_frame_constraint_fragment).
+
+
+
 		 /*******************************
 		 *	       EVENTS		*
 		 *******************************/
